#!/bin/sh
# COSMO applications boot and development script executed on TS7600
# script reloaded at each update
BTRACE=/root/boot_trace
BREQUEST=/tmp/boot_request
FREQUEST=/tmp/factory_request
APPNAME=/root/APPname
USRBIN=/usr/local/bin
MAKE=0

if test "$1" = "help"
then
	echo "COSMO applications boot and development script"
	echo "      parameters: \$1= start | stop         by reboot command"
	echo "                      make  | clean        by user for compile or clean compile"
	echo "                      prod                 by user for compile appli in production + 'only'"
	echo "                      only                 by user for delete all other appli"
	echo "                  \$2= appli name (CIT3 | CAT3 | CET3 | CTC3 ...)"
	echo "                      if not exists, see name in 'APPname' file"
	exit 0
elif test "$1" = "prod"
then
	$0 "clean" $2
	$0 "make" $2
fi

tshwctl --redledon >/dev/null 2>&1
cd /root
date +"%F %T COSMO Script "$1" "$2" ..." >$BTRACE

# Only at first start
if test "$1" = "start"
then
	export PATH=$PATH:/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin
	rm -f $BREQUEST >/dev/null 2>&1
	tshwctl --address=0x2e --poke16=0x2
	ifconfig eth0 >/dev/null 2>&1
	if test $? -eq 1
	then
		echo "TS7600 ERROR Eth0 not found " >>$BTRACE
		rm -f /etc/udev/rules.d/70-persistent-net.rules >/dev/null 2>&1
		tshwctl --greenledoff >/dev/null 2>&1
	fi
fi

# Load and control parameter APPLI
if test "$2" = ""; then CXX3=$(cat $APPNAME); else CXX3=$2; fi
cxx=$(echo $CXX3 |tr -d 0-9 |tr A-Z a-z)
echo $CXX3" Appli "$cxx >>$BTRACE
if test ! -d $CXX3
then
	echo $CXX3" ERROR appli not found" >>$BTRACE
	cat $BTRACE
	exit 1
fi
mkdir TS7600LIB/log $CXX3/log >/dev/null 2>&1
# To be accessed via the web interface
chown -R root:root $CXX3/cfg
chmod -R ug+w $CXX3/cfg

# Develop trace
test "$1" != "start" && cat $BTRACE

if test "$1" = "make" -o "$1" = "prod"
then
# Save current APPLI
	echo $CXX3 >$APPNAME
# Redirect Apache to CXX3/www
	diff /etc/apache2/sites-enabled/myappli /root/$CXX3/cfg/site-available >/dev/null 2>&1
	if test $? -ne 0
	then
		echo $CXX3" Reconfig Apache..."
		cd /etc/apache2/sites-enabled
		rm -f myappli
		ln -s /root/$CXX3/cfg/site-available myappli
		cd /root
		apache2ctl restart
	fi
 fi

# Stop all applications programs CXX3
echo $CXX3" Kill all processes" >>$BTRACE
for dd in $(ls -d [CD]??3/bin)
do
	for ff in $(ls $dd)
	do
		echo $CXX3" Kill "$ff >>$BTRACE
		test "$ff" != "version.txt" -a -x $dd/$ff && killall -s9 $ff >/dev/null 2>&1
	done
done
for ff in $(ls TS7600LIB/bin)
do
	test "$ff" != "status" -a -x TS7600LIB/bin/$ff && killall -s9 $ff >/dev/null 2>&1
done

# Exit if command stop
test "$1" = "stop" && exit 0

# Start UART
if ! test -h /dev/ttydisplay
then
	echo $CXX3" Mount 4 tty ..." >>$BTRACE
	eval $(/usr/bin/xuartctl --server --port 0 --speed 57600 2>&1); ln -s $ttyname /dev/ttydisplay
	eval $(/usr/bin/xuartctl --server --port 1 --speed 19200 --mode 8e1 2>&1); ln -s $ttyname /dev/ttycact
	eval $(/usr/bin/xuartctl --server --port 3 --speed 9600 2>&1); ln -s $ttyname /dev/ttybarcode
	eval $(/usr/bin/xuartctl --server --port 4 --speed 9600 2>&1); ln -s $ttyname /dev/ttyreader
fi

# Develop Make
if test "$1" = "make"
then
	if test "$CXX3" = "CAT3"
	then
		echo FPDF" Start make"
		mkdir FPDF153LIB/obj >/dev/null 2>&1
		mkdir FPDF153LIB/bin >/dev/null 2>&1
		cd /root/FPDF153LIB/src
		make
		cd /root
	fi
	echo $CXX3" Start make"
	mkdir TS7600LIB/obj $CXX3/obj >/dev/null 2>&1
	mkdir TS7600LIB/bin $CXX3/bin >/dev/null 2>&1
	mkdir $CXX3/dat >/dev/null 2>&1
	mkdir $CXX3/dat/hlp >/dev/null 2>&1
	cd /root/$CXX3
	make
	MAKE=$?
	cd /root
# Develop Clean
elif test "$1" = "clean"
then	
	cd /root/$CXX3
	make clean
	cd /root/FPDF153LIB/src
	make clean
	MAKE=1
	cd /root
# Install last update from APP/file.tar.gz or _tar_gz
else
	echo $CXX3" Search update "$cxx"xxx.tar.gz" >>$BTRACE
	ls $CXX3/$cxx???[._]tar[._]gz >/dev/null 2>&1
	if test $? -eq 0
	then
		for i in $(ls $CXX3/$cxx???[._]tar[._]gz)
		do
			echo $CXX3" found "$i >>$BTRACE
			mv -f $i $Cxx_last.tar.gz >>$BTRACE 2>&1
		done
		if test -f $cxx_last.tar.gz
		then
			echo $CXX3" Install update..." >>$BTRACE
			tar -f $cxx_last.tar.gz -xPzv >>$BTRACE 2>&1
			rm -f $cxx_last.tar* >>$BTRACE 2>&1
		fi
	fi
fi

# If param 'prod' delete all other applications and sources for production
if test "$1" = "only" -o "$1" = "prod"
then
	echo $CXX3" Remove sources and other app..."
	for dd in $(ls -d C??3)
	do
		test $dd != $CXX3 && rm -rf $dd
	done
	rm -rf $CXX3/obj $CXX3/src >/dev/null 2>&1
	rm -f $CXX3/Makefile ts76bin.tar.gz >/dev/null 2>&1
	rm -rf TS7600LIB/obj TS7600LIB/src >/dev/null 2>&1
	rm -rf FPDF153LIB/obj FPDF153LIB/src >/dev/null 2>&1
	rm -f c*.tar.gz >/dev/null 2>&1
# Remove entire DB if CAT3
	if test "$CXX3" = "CAT3"
	then
		read -p "Do you want to delete the entire DB? (confirm with 'yes') " ANSWER
		if test "$ANSWER" = "yes"
		then
			echo $CXX3" Remove entire DB..."
			rm -fR $CXX3/dat/acc $CXX3/dat/clk $CXX3/dat/prs >>$BTRACE 2>&1
			rm -f $CXX3/cfg/catman_cfg >>$BTRACE 2>&1
			touch $FREQUEST
		fi
	fi
# Sort config file
#	cat $CXX3/cfg/term_cfg |sort >/tmp/term_cfg
#	mv -f /tmp/term_cfg $CXX3/cfg/term_cfg
fi
# End prod

# Start only if make is OK
if test $MAKE -eq 0
then
# Start DATA I/O Server
	if ! test -h /var/run/dio_socket
	then
		echo $CXX3" Start dio server ..." >>$BTRACE
		/root/TS7600LIB/bin/ts76dio >>$BTRACE 2>&1 &
	fi

# Start ROOT ADMIN Server
	if ! test -h /var/run/adm_socket
	then
		echo $CXX3" Start admin server ..." >>$BTRACE
		/root/TS7600LIB/bin/ts76adm >>$BTRACE 2>&1 &
	fi

# Start LOGGING Server
	if ! test -h /var/run/log_socket
	then
		echo $CXX3" Start logging server ..." >>$BTRACE
		/root/TS7600LIB/bin/ts76log >>$BTRACE 2>&1 &
	fi

# Start Application
	echo $CXX3" Start config ..." >>$BTRACE
	/root/$CXX3/bin/$cxx"cfg" >/dev/null 2>&1 &
	if test "$CXX3" = "CAT3"
	then
		echo $CXX3" Start catman ..." >>$BTRACE
		mkdir $CXX3/dat/acc $CXX3/dat/clk $CXX3/dat/prs >/dev/null 2>&1
		/root/$CXX3/bin/catman >/dev/null 2>&1 &
	elif test "$CXX3" = "CIT3"
	then
		rm -f /root/$CXX3/bin/citupd >/dev/null 2>&1
	fi
fi

# If make is OK, prepare binary files for sending at next exchange
if test "$1" = "make" -a $MAKE -eq 0 -a -f $CXX3/bin/version.txt
then
	echo $CXX3" Compact binary files and return..."
	TAR_FILE=$(cat $CXX3/bin/version.txt)".tar.gz"
	rm -f $CXX3/$cxx???.tar.gz >/dev/null 2>&1
	tar -f $CXX3/$TAR_FILE -cPz TS7600LIB/bin FPDF153LIB/bin $CXX3/bin $CXX3/www boot_trace cosmo.sh
	tar -f ts76bin.tar.gz -cPz $CXX3/$TAR_FILE
fi

tshwctl --redledoff >/dev/null 2>&1
exit $MAKE

